package org.apidesign.apifest08.currency;

import java.util.*;

/**
 * ConvertorProvider class is introduced to handle
 */
public final class OfflineConverterProvider implements CurrencyConverterProvider {


	// we can't synchronize on DEFAULT as it's not final and can even be null in synchronized section
	private static final Object LOCK = new Object();
	private static CurrencyConverterProvider DEFAULT;

	// this is not nice and could be static, but future usage is uncertain and it's pretty easy to create
	private final Map<Currency, Double> rates;

	private OfflineConverterProvider() {
		rates = new HashMap<Currency, Double>();
		// simple initialization just for Task1Test
		rates.put(Currency.getInstance("USD"), 100.0);
		rates.put(Currency.getInstance("CZK"), 1700.0);
		rates.put(Currency.getInstance("SKK"), 2125.0);
	}

	/**
	 * Provides default implementation of ConvertorProvider. This Converter does nos not ensure accuracy
	 * of exchange rates, but should be functional at any circumstances including being offline.
	 *
	 * @return
	 */
	public static CurrencyConverterProvider getInstance() {
		// should be necessary in current implementation as creating CurrencyConverterProvider is cheap, but for future
		if (DEFAULT == null)
			synchronized (LOCK) {
				if (DEFAULT == null)
					DEFAULT = new OfflineConverterProvider();
			}
		return DEFAULT;
	}

	@Override
	public CurrencyConverter getConverter(double amount1, /*@NotNull*/ String currency1,
									   double amount2, /*@NotNull*/ String currency2)
			throws IllegalArgumentException {

		return getConverter(amount1, Currency.getInstance(currency2), amount2, Currency.getInstance(currency1));
	}

	/**
	 * Retrieves converter that is capable of converting values between currency1 and currency2. The exchange is
	 * specified in easy to understand way. By specifying values in two currencies that are equal. For example
	 * CurrencyConverter.getConverter(25, "CZK", 1, "EUR"); means 25CKZ is equal to 1EUR. This enables user to
	 * use this method without having to calculate anything. In general this can be expressed by formula
	 * amount1[currency1] = amount2[currency2].
	 *
	 * @param amount1
	 * @param currency1
	 * @param amount2
	 * @param currency2
	 *
	 * @return
	 *
	 * @throws IllegalArgumentException
	 * @throws CurrencyNotAvailableException
	 *
	 */
	@Override
	public CurrencyConverter getConverter(double amount1, /*@NotNull*/ Currency currency1,
									   double amount2, /*@NotNull*/ Currency currency2)
			throws IllegalArgumentException, CurrencyNotAvailableException {
		if (amount1 <= 0.0 || amount2 <= 0.0)
			throw new IllegalArgumentException(
					String.format("The specified currency values (%1$s, %2$s)", amount1, amount2));
		return new ConstantRateConverter(currency2, currency1, amount2 / amount1);
	}

	/**
	 * Creates a new converter that is able to convert between all specified currencies.
	 *
	 * @param currencies that the converter should be created for
	 *
	 * @return converter able to convert between all specified currencies
	 *
	 * @throws CurrencyNotAvailableException
	 *          thrown when one of the currencies is not available
	 */
	@Override
	public CurrencyConverter getConverter(/*@NotNull*/ Currency... currencies) throws CurrencyNotAvailableException {
		for (Currency c : currencies) {
			if (c == null)
				throw new NullPointerException("One of the specified currencies is null");
			if (!rates.containsKey(c))
				throw new CurrencyNotAvailableException(c);
		}
		return new MultiCurrencyConstantRateConverter(rates);
	}

	/**
	 * Creates a new converter that is able to convert between all specified currencies.
	 *
	 * @param currencies that the converter should be created for
	 *
	 * @return converter able to convert between all specified currencies
	 *
	 * @throws CurrencyNotAvailableException
	 *                                  thrown when one of the currencies is not available
	 * @throws NullPointerException	 if any of the specified currencies is null
	 * @throws IllegalArgumentException if any of the specified currencies is not a valid ISO code
	 */
	@Override
	public CurrencyConverter getConverter(/*@NotNull*/ String... currencies)
			throws CurrencyNotAvailableException, IllegalArgumentException, NullPointerException {

		Currency[] c2 = new Currency[currencies.length];
		for (int i = 0; i < c2.length; i++)
			c2[i] = Currency.getInstance(currencies[i]);
		return getConverter(c2);
	}
}
